@@ -11,6 +11,7 @@
 ******************************************************************************/
 
 #include "Tex/convert_tex.hpp"
+#include "converter.hpp"
 
 /******************************************************************************
 * The latex_parser structure
@@ -39,10 +40,11 @@
 ******************************************************************************/
 
 struct latex_parser {
-  latex_parser () {}
+  bool unicode;
+  latex_parser (bool unicode2): unicode (unicode2) {}
   void latex_error (string s, int i, string message);
 
-  tree parse           (string s, int& i, char stop= '\0', bool change= false);
+  tree parse           (string s, int& i, string stop= "", bool change= false);
   tree parse_backslash (string s, int& i);
   tree parse_symbol    (string s, int& i);
   tree parse_command   (string s, int& i, string which);
@@ -68,8 +70,16 @@ latex_parser::latex_error (string s, int i, string message) {
 * Main parsing routine
 ******************************************************************************/
 
+static bool
+is_regular (tree t) {
+  if (!is_tuple (t)) return true;
+  if (N(t) == 0 || !is_atomic (t[0])) return false;
+  string s= t[0]->label;
+  return !starts (s, "\\begin-") && !starts (s, "\\end-");
+}
+
 tree
-latex_parser::parse (string s, int& i, char stop, bool change) {
+latex_parser::parse (string s, int& i, string stop, bool change) {
   bool no_error= true;
   int n= N(s);
   tree t (CONCAT);
@@ -79,15 +89,20 @@ latex_parser::parse (string s, int& i, char stop, bool change) {
   command_def  ->extend ();
 
   while ((i<n) && is_space (s[i])) i++;
-  while ((i<n) && (s[i]!=stop) && no_error) {
+  while ((i<n) && no_error &&
+	 (s[i] != '\0' || N (stop) != 0) &&
+	 (N(stop) != 1 || s[i] != stop[0]) &&
+	 (s[i] != '$' || stop != "$$" || i+1>=n || s[i+1] != '$')) {
     switch (s[i]) {
     case '~':
-      t << tuple ("\\nbsp");
+      if (command_type ["!mode"] == "math") t << tuple ("\\sim");
+      else t << tuple ("\\nbsp");
       i++;
       break;
     case ' ':
     case '\t':
-      while ((i<n) && ((s[i]==' ') || (s[i]=='\t'))) i++;
+    case '\r':
+      while ((i<n) && ((s[i]==' ') || (s[i]=='\t') || (s[i]=='\r'))) i++;
       if ((i<n) && (s[i]!='\n')) t << " ";
       break;
     case '\n': {
@@ -100,25 +115,19 @@ latex_parser::parse (string s, int& i, char stop, bool change) {
       }
       break;
     }
-    case '%':
-      if (test (s, i, "%%%%%%%%%% Start TeXmacs macros\n")) {
-	while ((i<n) && (!test (s, i, "%%%%%%%%%% End TeXmacs macros\n")))
-	  i++;
-	i += 30;
-      }
-      else {
-	while ((i<n) && (s[i]!='\n')) i++;
-	if (i<n) i++;
-	int ln=0;
-	while ((i<n) && ((s[i]==' ') || (s[i]=='\t') || (s[i]=='\n')))
-	  if (s[i++]=='\n') ln++;
-	if (ln > 0) {
-	  if ((N(t)>0) && ((t[N(t)-1]==" ") || (t[N(t)-1]=="\n")))
-	    t[N(t)-1]= "\n";
-	  else t << "\n";
-	}
+    case '%': {
+      while ((i<n) && (s[i]!='\n')) i++;
+      if (i<n) i++;
+      int ln=0;
+      while ((i<n) && is_space (s[i]))
+	if (s[i++]=='\n') ln++;
+      if (ln > 0) {
+	if ((N(t)>0) && ((t[N(t)-1]==" ") || (t[N(t)-1]=="\n")))
+	  t[N(t)-1]= "\n";
+	else t << "\n";
       }
       break;
+    }
     case '#':
       i++;
       if (i==n) return t;
@@ -130,13 +139,16 @@ latex_parser::parse (string s, int& i, char stop, bool change) {
       break;
     case '\\':
       if (((i+7)<n) && (s(i,i+5)=="\\over") && (!is_alpha (s (i+5, i+7)))) {
+	int j;
 	i+=5;
 	tree arg= parse_command (s, i, "\\over");
-	while ((N(t)>=1) &&
-	       ((t[N(t)-1] == " ") || (t[N(t)-1] == tree (TUPLE, "\\ "))))
-	  t= t (0, N(t)-1);
-	if (is_tuple (arg, "\\over", 1) && (N(t)>=1))
-	  t[N(t)-1]= tree (TUPLE, "\\frac", t[N(t)-1], arg[1]);
+	for (j=N(t); j>0 && is_regular (t[j-1]); j--);
+	if (is_tuple (arg, "\\over", 1)) {
+	  tree num= t (j, N(t));
+	  if (N(num) == 0) num= "";
+	  t= t (0, j);
+	  t << tree (TUPLE, "\\frac", num, arg[1]);
+	}
       }
       else if (((i+5)<n) && (s(i,i+3)=="\\sp") && (!is_alpha(s[i+3]))) {
 	i+=3;
@@ -206,45 +218,51 @@ latex_parser::parse (string s, int& i, char stop, bool change) {
       break;
     case '{': {
       i++;
-      t << parse (s, i, '}');
+      t << parse (s, i, "}");
       if ((i<n) && (s[i]=='}')) i++;
 
       int ln=0;
-      if ((i<n) && (s[i]!=' ') && (s[i]!='\t') && (s[i]!='\n')) break;
-      while ((i<n) && ((s[i]==' ') || (s[i]=='\t') || (s[i]=='\n')))
+      if ((i<n) && (!is_space (s[i]))) break;
+      while ((i<n) && is_space (s[i]))
 	if (s[i++]=='\n') ln++;
       if (ln >= 2) t << "\n"; else t << tree (TUPLE, "\\ ");
       break;
     }
     case '$': {
-      bool flag= false;
       i++;
       if ((i<n) & (s[i]=='$')) {
 	i++;
-	flag=true;
 	t << tree (TUPLE, "\\begin-displaymath");
-      }
-      else t << tree (TUPLE, "\\begin-math");
-      command_type ("!mode")= "math";
-      t << parse (s, i, '$');
-      command_type ("!mode")= "text";
-      if ((i<n) && (s[i]=='$')) i++;
-      if (flag) {
+	command_type ("!mode")= "math";
+	t << parse (s, i, "$$");
+	command_type ("!mode")= "text";
+	if ((i<n) && (s[i]=='$')) i++;
 	if ((i<n) && (s[i]=='$')) i++;
 	t << tree (TUPLE, "\\end-displaymath");
       }
-      else t << tree (TUPLE, "\\end-math");
+      else {
+	t << tree (TUPLE, "\\begin-math");
+	command_type ("!mode")= "math";
+	t << parse (s, i, "$");
+	command_type ("!mode")= "text";
+	if ((i<n) && (s[i]=='$')) i++;
+	t << tree (TUPLE, "\\end-math");
+      }
       break;
     }
     default:
-      if (is_iso_alpha (s[i])) {
+      if (unicode && ((unsigned char) s[i]) >= 128) {
+	unsigned int code= decode_from_utf8 (s, i);
+	t << tree (TUPLE, "\\#" * as_hexadecimal (code));
+      }
+      else if (!unicode && is_iso_alpha (s[i])) {
 	// If we encounter too much text in math mode, then return
 	int start= i;
 	while ((i<n) && is_iso_alpha (s[i])) i++;
 	int end= i;
 	if ((i >= start+3) && (command_type ["!mode"] == "math")) {
 	  while ((i<n) && (is_iso_alpha (s[i]) ||
-			   is_ponctuation (s[i]) ||
+			   is_punctuation (s[i]) ||
 			   is_space (s[i])))
 	    i++;
 	  if (i >= start+20) {
@@ -304,6 +322,33 @@ latex_parser::parse_backslash (string s, int& i) {
     i+=16;
     return parse_verbatim (s, i, "\\end{verbatim}");
   }
+  if (((i+5)<n) && (s(i,i+4)=="\\url")) {
+    i+=4;
+    while (i<n && (s[i] == ' ' || s[i] == '\n' || s[i] == '\t')) i++;
+    string ss;
+    if (i<n && s[i] == '{') {
+      i++;
+      int start= i;
+      while ((i<n) && s[i] != '}') i++;
+      ss= s (start, i++);
+    }
+    return tree (TUPLE, "\\url", ss);
+  }
+  if (((i+6)<n) && (s(i,i+5)=="\\href")) {
+    i+=5;
+    while (i<n && (s[i] == ' ' || s[i] == '\n' || s[i] == '\t')) i++;
+    string ss;
+    if (i<n && s[i] == '{') {
+      i++;
+      int start= i;
+      while ((i<n) && s[i] != '}') i++;
+      ss= s (start, i++);
+    }
+    tree u= "";
+    while (i<n && (s[i] == ' ' || s[i] == '\n' || s[i] == '\t')) i++;
+    if (i<n && s[i] == '{') { i++; u= parse (s, i, "}"); i++; }
+    return tree (TUPLE, "\\href", ss, u);
+  }
 
   /************************ special commands *********************************/
   i++;
@@ -324,10 +369,10 @@ latex_parser::parse_backslash (string s, int& i) {
   /************************* normal commands *********************************/
   int start= i-1;
   while ((i<n) && is_alpha (s[i])) i++;
-  if ((i<n) && (s[i]=='*')) i++;
+  if ((i<n) && (s[i]=='*') && latex_type (s (start, i+1)) != "undefined") i++;
   string r= s (start, i);
   if ((r == "\\begin") || (r == "\\end")) {
-    while ((i<n) && ((s[i]==' ') || (s[i]=='\t') || (s[i]=='\n'))) i++;
+    while ((i<n) && is_space (s[i])) i++;
     if ((i==n) || (s[i]!='{')) {
       latex_error (s, i, "begin or end which environment ?");
       return "";
@@ -372,7 +417,7 @@ latex_parser::parse_symbol (string s, int& i) {
 tree
 latex_parser::parse_command (string s, int& i, string cmd) {
   /*
-  cout << cmd << " [" << command_type [cmd] << ", "
+  cout << cmd << " [" << latex_type (cmd) << ", "
        << command_type ["!mode"] << "]" << LF;
   */
   if (cmd == "\\newcommand") cmd= "\\def";
@@ -382,11 +427,10 @@ latex_parser::parse_command (string s, int& i, string cmd) {
   if (cmd == "\\end-split") cmd= "\\end-eqsplit";
   if (cmd == "\\begin-split*") cmd= "\\begin-eqsplit*";
   if (cmd == "\\end-split*") cmd= "\\end-eqsplit*";
-  if ((!command_type->contains (cmd)) &&
-      (latex_type [cmd] == "undefined"))
+  if (latex_type (cmd) == "undefined")
     return parse_unknown (s, i, cmd);
 
-  if (latex_type [cmd] == "math-environment") {
+  if (latex_type (cmd) == "math-environment") {
     if (cmd (0, 6) == "\\begin") command_type ("!mode") = "math";
     else command_type ("!mode") = "text";
   }
@@ -397,22 +441,21 @@ latex_parser::parse_command (string s, int& i, string cmd) {
   if (mbox_flag) command_type ("!mode") = "text";
 
   int  n     = N(s);
-  int  arity =
-    (latex_type [cmd]=="undefined")? command_arity (cmd): latex_arity [cmd];
+  int  arity = latex_arity (cmd);
   bool option= (arity<0);
   if (option) arity= -1-arity;
 
   /************************ retrieve arguments *******************************/
   tree t (TUPLE, copy (cmd)); // parsed arguments
   tree u (TUPLE, copy (cmd)); // unparsed arguments
-  while ((i<n) && ((arity>0) || option)) {
-    int j=i;
-    while ((j<n) && ((s[j]==' ') || (s[j]=='\t') || (s[j]=='\n'))) j++;
+  while (i<n && arity>=0 && (arity>0 || option)) {
+    int j= i;
+    while ((j<n) && is_space (s[j])) j++;
     if (j==n) break;
     if (option && (s[j]=='[')) {
       j++;
       i=j;
-      t << parse (s, i, ']');
+      t << parse (s, i, "]");
       u << s (j, i);
       if ((i<n) && (s[i]==']')) i++;
       t[0]->label= t[0]->label * "*";
@@ -425,10 +468,11 @@ latex_parser::parse_command (string s, int& i, string cmd) {
 	while ((i<n) && (s[i]!='}')) i++;
 	t << s (j, i);
       }
-      else t << parse (s, i, '}');
+      else t << parse (s, i, "}");
       u << s (j, i);
       if ((i<n) && (s[i]=='}')) i++;
       arity--;
+      if (arity == 0) option= false;
     }
     else if (option && (s[j]=='#') && (cmd == "\\def")) {
       while ((j+3 <= n) && is_numeric (s[j+1]) && (s[j+2] == '#')) j+=2;
@@ -447,6 +491,7 @@ latex_parser::parse_command (string s, int& i, string cmd) {
 	t << st;
 	u << st;
 	arity--;
+	if (arity == 0) option= false;
       }
       else break;
     }
@@ -488,11 +533,15 @@ latex_parser::parse_command (string s, int& i, string cmd) {
   }
 
   /***************** environment changes for user commands  ******************/
-  if (command_type[cmd] == "user") {
+  if (latex_type (cmd) == "user") {
     int pos= 0;
-    (void) parse (sharp_to_arg (command_def[cmd], u), pos, '\0', true);
-    // t= parse (sharp_to_arg (command_def[cmd], u), pos, '\0', true);
-    // variant if you want to replace macros by their definitions
+    string body= command_def[cmd];
+    if (count_occurrences ("\\begin", body) ==
+	count_occurrences ("\\end", body))
+      (void) parse (sharp_to_arg (body, u), pos, "", true);
+    else t= parse (sharp_to_arg (body, u), pos, "", true);
+    // replaces macros by their definitions in the case when
+    // the user defined shorthands for \\begin{env} and \\end{env}
   }
 
   if (mbox_flag) command_type ("!mode") = "math";
@@ -507,12 +556,12 @@ latex_parser::parse_unknown (string s, int& i, string cmd) {
   tree t (TUPLE, copy (cmd));
   while (i<n) {
     int j=i;
-    while ((j<n) && ((s[j]==' ') || (s[j]=='\t') || (s[j]=='\n'))) j++;
+    while ((j<n) && is_space (s[j])) j++;
     if (j==n) break;
     if (option && (s[j]=='[')) {
       j++;
       i=j;
-      t << parse (s, i, ']');
+      t << parse (s, i, "]");
       if ((i<n) && (s[i]==']')) i++;
       t[0]->label= t[0]->label * "*";
       option= false;
@@ -520,7 +569,7 @@ latex_parser::parse_unknown (string s, int& i, string cmd) {
     else if (s[j]=='{') {
       j++;
       i=j;
-      t << parse (s, i, '}');
+      t << parse (s, i, "}");
       if ((i<n) && (s[i]=='}')) i++;
     }
     else break;
@@ -548,11 +597,11 @@ latex_parser::parse_verbatim (string s, int& i, string end) {
 ******************************************************************************/
 
 static char Cork_unaccented[128]= {
-  'A', ' ', 'C', 'C', 'D', 'E', ' ', 'G',
+  'A', 'A', 'C', 'C', 'D', 'E', 'E', 'G',
   'L', 'L', ' ', 'N', 'N', ' ', 'O', 'R',
   'R', 'S', 'S', 'S', 'T', 'T', 'U', 'U',
   'Y', 'Z', 'Z', 'Z', ' ', 'I', 'd', ' ',
-  'a', ' ', 'c', 'c', 'd', 'e', ' ', 'g',
+  'a', 'a', 'c', 'c', 'd', 'e', 'e', 'g',
   'l', 'l', ' ', 'n', 'n', ' ', 'o', 'r',
   'r', 's', 's', 's', 't', 't', 'u', 'u',
   'y', 'z', 'z', 'z', ' ', ' ', ' ', ' ',
@@ -561,19 +610,19 @@ static char Cork_unaccented[128]= {
   'D', 'N', 'O', 'O', 'O', 'O', 'O', ' ',
   ' ', 'U', 'U', 'U', 'U', 'Y', ' ', ' ',
   'a', 'a', 'a', 'a', 'a', 'a', ' ', 'c',
-  'e', 'e', 'e', 'e', 16 , 16 , 16 , 16 ,
+  'e', 'e', 'e', 'e', 25 , 25 , 25 , 25 ,
   'd', 'n', 'o', 'o', 'o', 'o', 'o', ' ',
   ' ', 'u', 'u', 'u', 'u', 'y', ' ', ' '
 };
 
 static char Cork_accent[128]= {
-  'u' , ' ' , '\'', 'v' , 'v' , 'v' , ' ' , 'u' ,
-  '\'', ' ' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
-  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , ' ' ,
+  'u' , 'k' , '\'', 'v' , 'v' , 'v' , 'k' , 'u' ,
+  '\'', 'v' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
+  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , 'r' ,
   '\"', '\'', 'v' , '.' , ' ' , '.' , '=' , ' ' , // "
-  'u' , ' ' , '\'', 'v' , 'v' , 'v' , ' ' , 'u' ,
-  '\'', ' ' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
-  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , ' ' ,
+  'u' , 'k' , '\'', 'v' , 'v' , 'v' , 'k' , 'u' ,
+  '\'', 'v' , ' ' , '\'', 'v' , ' ' , 'H' , '\'',
+  'v' , '\'', 'v' , 'c' , 'v' , 'c' , 'H' , 'r' ,
   '\"', '\'', 'v' , '.' , ' ' , ' ' , ' ' , ' ' , // "
   '`' , '\'', '^' , '~' , '\"', ' ' , ' ' , 'c' , // "
   '`' , '\'', '^' , '\"', '`' , '\'', '^' , '\"', // "
@@ -595,8 +644,8 @@ accented_to_Cork (tree t) {
 
   string s= t[0]->label;
   if ((N(s)==2) && (s[0]=='\\') && (n==2) &&
-      is_atomic (t[1]) && (N(t[1]->label)<=2)) {
-    string v= t[1]->label;
+      is_atomic (r[1]) && (N(r[1]->label)<=2)) {
+    string v= r[1]->label;
     if (N(v)==0) {
       if (s[1] == '`' ) {
 	string ret_s (1);
@@ -616,13 +665,14 @@ accented_to_Cork (tree t) {
     }
     else {
       char c1= v[0], c2= s[1];
-      if (v=="\\i") c1= (char) 16;
+      if (v == "\\i") c1= (char) 25;
       if ((N(v)==1) || (v=="\\i"))
 	for (i=0; i<127; i++)
 	  if ((Cork_unaccented[i]==c1) && (Cork_accent[i]==c2))
 	    return tree (string ((char) (i+128)));
     }
   }
+  if (r == tuple ("\\i")) return "\\i";
   return r;
 }
 
@@ -642,7 +692,15 @@ latex_parser::parse (string s) {
   int i, start=0, n= N(s);
   for (i=0; i<n; i++)
     if (s[i]=='\n') {
-      while ((i<n) && ((s[i]=='\12') || (s[i]=='\15') || (s[i]==' '))) i++;
+      while ((i<n) && is_space (s[i])) i++;
+      if (test (s, i, "%%%%%%%%%% Start TeXmacs macros\n")) {
+	a << s (start, i);
+	while ((i<n) && (!test (s, i, "%%%%%%%%%% End TeXmacs macros\n")))
+	  i++;
+	i += 30;
+	start= i;
+	continue;
+      }
       if (test (s, i, "\\begin{document}") ||
 	  test (s, i, "\\begin{abstract}") ||
 	  test (s, i, "\\chapter") ||
@@ -657,6 +715,7 @@ latex_parser::parse (string s) {
 	  a << s (start, i);
 	  start= i;
 	}
+      if (i == n) break;
     }
   a << s (start, i);
 
@@ -668,7 +727,7 @@ latex_parser::parse (string s) {
       int start= j;
       command_type ("!mode") = "text";
       command_type ("!em") = "false";
-      tree u= parse (a[i], j, '\0', true);
+      tree u= parse (a[i], j, "", true);
       if ((N(t)>0) && (t[N(t)-1]!='\n') && (start==0)) t << "\n";
       if (is_concat (u)) t << A(u);
       else t << u;
@@ -682,11 +741,120 @@ latex_parser::parse (string s) {
   return t;
 }
 
+static bool
+japanese_tex (string& s) {
+  if (search_forwards ("\\documentclass{jarticle}", s) != -1) {
+    s= replace (s, "\\documentclass{jarticle}", "\\documentclass{article}");
+    s= convert (s, "ISO-2022-JP", "UTF-8");
+    return true;
+  }
+  if (search_forwards ("\\documentclass{jbook}", s) != -1) {
+    s= replace (s, "\\documentclass{jbook}", "\\documentclass{book}");
+    s= convert (s, "ISO-2022-JP", "UTF-8");
+    return true;
+  }
+  return false;
+}
+
+static bool
+korean_tex (string& s) {
+  if (search_forwards ("\\usepackage{hangul}", s) != -1 ||
+      search_forwards ("\\usepackage{hfont}", s) != -1 ||
+      search_forwards ("]{hangul}", s) != -1 ||
+      search_forwards ("]{hfont}", s) != -1)
+    {
+      s= replace (s, "\\usepackage{hangul}", "");
+      s= replace (s, "\\usepackage{hfont}", "");
+      s= convert (s, "EUC-KR", "UTF-8");
+      return true;
+    }
+  if (search_forwards ("\\usepackage{dhucs}", s) != -1 ||
+      search_forwards ("\\usepackage{memhangul-ucs}", s) != -1 ||
+      search_forwards ("]{dhucs}", s) != -1 ||
+      search_forwards ("]{memhangul-ucs}", s) != -1)
+    {
+      s= replace (s, "\\usepackage{dhucs}", "");
+      s= replace (s, "\\usepackage{memhangul-ucs}", "");
+      return true;
+    }
+  return false;
+}
+
+static bool
+chinese_tex (string& s) {
+  if (search_forwards ("\\kaishu", s) != -1)
+    s= replace (s, "\\kaishu", "");
+  if (search_forwards ("\\begin{CJK}{GBK}{kai}", s) != -1)
+    s= replace (s, "\\begin{CJK}{GBK}{kai}", "");
+  if (search_forwards ("\\begin{CJK*}{GBK}{kai}", s) != -1)
+    s= replace (s, "\\begin{CJK*}{GBK}{kai}", "");
+  if (search_forwards ("\\end{CJK}", s) != -1)
+    s= replace (s, "\\end{CJK}", "");
+  if (search_forwards ("\\end{CJK*}", s) != -1)
+    s= replace (s, "\\end{CJK*}", "");
+  if (search_forwards ("\\CJKindent", s) != -1)
+    s= replace (s, "\\CJKindent", "");
+  if (search_forwards ("\\CJKcaption{GBk}", s) != -1)
+    s= replace (s, "\\CJKcaption{GBK}", "");
+  if (search_forwards ("\\usepackage{CJK}", s) != -1) {
+    s= replace (s, "\\usepackage{CJK}", "");
+    s= convert (s, "cp936", "UTF-8");
+    return true;
+  }
+  if (search_forwards ("\\documentclass{cctart}", s) != -1) {
+    s= replace (s, "\\documentclass{cctart}", "\\documentclass{article}");
+    s= convert (s, "cp936", "UTF-8");
+    return true;
+  }
+  if (search_forwards ("\\documentclass[CJK]{cctart}", s) != -1) {
+    s= replace (s, "\\documentclass[CJK]{cctart}", "\\documentclass{article}");
+    s= convert (s, "cp936", "UTF-8");
+    return true;
+  }
+  return false;
+}
+
+static bool
+taiwanese_tex (string& s) {
+  if (search_forwards ("\\usepackage{CJKvert,type1cm}", s) != -1)
+    s= replace (s, "\\usepackage{CJKvert,type1cm}", "");
+  if (search_forwards ("\\begin{CJK}{Bg5}{aming}", s) != -1)
+    s= replace (s, "\\begin{CJK}{Bg5}{aming}", "");
+  if (search_forwards ("\\begin{CJK}{Bg5}{kai}", s) != -1)
+    s= replace (s, "\\begin{CJK}{Bg5}{kai}", "");
+  if (search_forwards ("\\end{CJK}", s) != -1)
+    s= replace (s, "\\end{CJK}", "");
+  if (search_forwards ("\\CJKcaption{Bg5}", s) != -1)
+    s= replace (s, "\\CJKcaption{Bg5}", "");
+  if (search_forwards ("\\CJKindent", s) != -1)
+    s= replace (s, "\\CJKindent", "");
+  if (search_forwards ("\\usepackage{CJK}", s) != -1) {
+    s= replace (s, "\\usepackage{CJK}", "");
+    s= convert (s, "cp950", "UTF-8");
+    return true;
+  }
+  if (search_forwards ("\\usepackage{CJK*}", s) != -1) {
+    s= replace (s, "\\usepackage{CJK*}", "");
+    s= convert (s, "cp950", "UTF-8");
+    return true;
+  }
+  return false;
+}
+
 tree
 parse_latex (string s) {
   s= dos_to_better (s);
-  latex_parser ltx;
-  return accented_to_Cork (ltx.parse (s));
+  string lan= "";
+  if (japanese_tex (s)) lan= "japanese";
+  else if (korean_tex (s)) lan= "korean";
+  else if (taiwanese_tex (s)) lan= "taiwanese";
+  else if (chinese_tex (s)) lan= "chinese";
+  bool unicode= (lan == "chinese" || lan == "japanese" ||
+		 lan == "korean" || lan == "taiwanese");
+  latex_parser ltx (unicode);
+  tree r= accented_to_Cork (ltx.parse (s));
+  if (lan == "") return r;
+  return compound ("!language", r, lan);
 }
 
 tree
