@@ -13,7 +13,15 @@
 #ifndef EDITOR_H
 #define EDITOR_H
 #include "typesetter.hpp"
+#ifdef AQUATEXMACS
+#include "Cocoa/aqua_simple_widget.h"
+#else
+#ifdef QTTEXMACS
+#include "Qt/qt_simple_widget.hpp"
+#else
 #include "Widkit/simple_wk_widget.hpp"
+#endif
+#endif
 #include "server.hpp"
 #include "scheme.hpp"
 #include "drd_info.hpp"
