@@ -75,6 +75,24 @@ texmacs_version (string which) {
   return TEXMACS_VERSION;
 }
 
+bool
+gui_is_x () {
+#ifdef QTTEXMACS
+  return false;
+#else
+  return true;
+#endif
+}
+
+bool
+gui_is_qt () {
+#ifdef QTTEXMACS
+  return true;
+#else
+  return false;
+#endif
+}
+
 bool
 os_win32 () {
 #ifdef OS_WIN32
@@ -342,7 +360,7 @@ tree_append (tree t1, tree t2) {
 bool
 tree_active (tree t) {
   path ip= obtain_ip (t);
-  return nil (ip) || last_item (ip) != DETACHED;
+  return is_nil (ip) || last_item (ip) != DETACHED;
 }
 
 tree
@@ -363,7 +381,7 @@ tree_child_insert (tree t, int pos, tree x) {
 
 static inline bool
 ip_attached (path ip) {
-  return nil (ip) || last_item (ip) != DETACHED;
+  return is_nil (ip) || last_item (ip) != DETACHED;
 }
 
 tree
@@ -570,7 +588,7 @@ scm_is_path (SCM p) {
 
 SCM
 path_to_scm (path p) {
-  if (nil (p)) return SCM_NULL;
+  if (is_nil (p)) return SCM_NULL;
   else return scm_cons (int_to_scm (p->item), path_to_scm (p->next));
 }
 
@@ -1124,7 +1142,7 @@ scm_is_list_string (SCM p) {
 
 SCM
 list_string_to_scm (list_string l) {
-  if (nil (l)) return SCM_NULL;
+  if (is_nil (l)) return SCM_NULL;
   return scm_cons (string_to_scm (l->item),
 		   list_string_to_scm (l->next));
 }
@@ -1151,7 +1169,7 @@ scm_is_list_tree (SCM p) {
 
 SCM
 list_tree_to_scm (list_tree l) {
-  if (nil (l)) return SCM_NULL;
+  if (is_nil (l)) return SCM_NULL;
   return scm_cons (tree_to_scm (l->item),
 		   list_tree_to_scm (l->next));
 }
@@ -1188,32 +1206,33 @@ scm_to_collection (SCM p) {
 
 void
 initialize_glue () {
-  tree_tag= scm_make_smob_type ("tree", 0);
+  tree_tag= scm_make_smob_type (const_cast<char*> ("tree"), 0);
   scm_set_smob_mark (tree_tag, mark_tree);
   scm_set_smob_free (tree_tag, free_tree);
   scm_set_smob_print (tree_tag, print_tree);
   scm_set_smob_equalp (tree_tag, cmp_tree);
-  observer_tag= scm_make_smob_type ("observer", 0);
+  observer_tag= scm_make_smob_type (const_cast<char*> ("observer"), 0);
   scm_set_smob_mark (observer_tag, mark_observer);
   scm_set_smob_free (observer_tag, free_observer);
   scm_set_smob_print (observer_tag, print_observer);
   scm_set_smob_equalp (observer_tag, cmp_observer);
-  widget_tag= scm_make_smob_type ("widget", 0);
+  widget_tag= scm_make_smob_type (const_cast<char*> ("widget"), 0);
   scm_set_smob_mark (widget_tag, mark_widget);
   scm_set_smob_free (widget_tag, free_widget);
   scm_set_smob_print (widget_tag, print_widget);
   scm_set_smob_equalp (widget_tag, cmp_widget);
-  promise_widget_tag= scm_make_smob_type ("promise-widget", 0);
+  promise_widget_tag=
+    scm_make_smob_type (const_cast<char*> ("promise-widget"), 0);
   scm_set_smob_mark (promise_widget_tag, mark_promise_widget);
   scm_set_smob_free (promise_widget_tag, free_promise_widget);
   scm_set_smob_print (promise_widget_tag, print_promise_widget);
   scm_set_smob_equalp (promise_widget_tag, cmp_promise_widget);
-  command_tag= scm_make_smob_type ("command", 0);
+  command_tag= scm_make_smob_type (const_cast<char*> ("command"), 0);
   scm_set_smob_mark (command_tag, mark_command);
   scm_set_smob_free (command_tag, free_command);
   scm_set_smob_print (command_tag, print_command);
   scm_set_smob_equalp (command_tag, cmp_command);
-  url_tag= scm_make_smob_type ("url", 0);
+  url_tag= scm_make_smob_type (const_cast<char*> ("url"), 0);
   scm_set_smob_mark (url_tag, mark_url);
   scm_set_smob_free (url_tag, free_url);
   scm_set_smob_print (url_tag, print_url);
